//
// Written by Igor Tolmachev, IT Samples
//        mailto:support@itsamples.com
//
// Copyright (c) 2009.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name and all copyright 
// notices remains intact. If the source code in this file is used in 
// any  commercial application then a statement along the lines of 
// "Portions copyright (c) Igor Tolmachev, 2009" must be included in 
// the startup banner, "About" box or printed documentation. An email 
// letting me know that you are using it would be nice as well. That's 
// not much to ask considering the amount of work that went into this.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
// Expect bugs!
// 
// Please use and enjoy, and let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into 
// this file. 

// TheBatStorage.cpp: implementation of the TheBatStorage class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TheBatRecovery.h"
#include "TheBatStorage.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

const char* g_pszTable = "+-0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTheBatStorage::CTheBatStorage()
{
	m_hWndParent = NULL;
}

CTheBatStorage::~CTheBatStorage()
{

}

void CTheBatStorage::SetMessageTarget(HWND hWnd)
{
	m_hWndParent = hWnd;
}

void CTheBatStorage::PostLastMessage(char* pszMessage)
{
	if(m_hWndParent)
		SendMessage(m_hWndParent, WM_POSTLASTMESSAGE, 0, reinterpret_cast<LPARAM>(pszMessage));
}

BOOL CTheBatStorage::LoadFromFile(char *pszPath)
{
	FILE* pAccount = fopen(pszPath, "rb");

	if(pAccount == NULL)
	{
		PostLastMessage("Failed to open selected file.");
		return FALSE;
	}

	fseek(pAccount, 0, SEEK_END);
	unsigned long nLen = ftell(pAccount);

	if(nLen)
	{
		PostLastMessage(pszPath);
		PostLastMessage("");

		unsigned char* lpBuffer = new unsigned char[nLen + 1];
		memset(lpBuffer, 0, nLen + 1);
		
		rewind(pAccount);
		fread(lpBuffer, nLen, 1, pAccount);

		char* pszDataBlock = new char[nLen + 1];
		memset(pszDataBlock, 0, nLen + 1);

		int nCounter = 0;
		bool bIsValidString = true;
		for(long i=nLen; i>0; i--)
		{
			if(lpBuffer[i] == 0x00)
			{
				if(strlen(pszDataBlock) > 5 && strlen(pszDataBlock) < 96) //usually less than 64
				{
					bIsValidString = true;
					char* pszRevData = strrev(pszDataBlock);
					
					for(unsigned long j=0; j<strlen(pszRevData); j++)
					{
						if(strchr(g_pszTable, pszRevData[j]) == 0)
							bIsValidString = false;
					}
					
					if(bIsValidString)
						Dump(pszRevData);
				}
				
				memset(pszDataBlock, 0, nLen + 1);
				nCounter = 0;
				continue;
			}

			pszDataBlock[nCounter] = lpBuffer[i];
			nCounter++;
		}

		delete [] pszDataBlock;
		delete [] lpBuffer;
	}

	fclose(pAccount);
	
	return TRUE;
}

void CTheBatStorage::Dump(char* pszPassword)
{
	int i=0;
	int b = 11;
	int z = 0;

	int nPos = 0;
	int nLoopCount = 0;
	int nIndex = 0;
	BOOL bIsValidString = TRUE;

	char szDecrypted[512];
	memset(szDecrypted, 0, 512);
	
	while(i<b)
	{
		if(nLoopCount>100)
		{
			bIsValidString = FALSE;
			break;
		}
		
		if(i >= (int)strlen(pszPassword) - 3)
		{
			bIsValidString = FALSE;
			break;
		}

		char* pI = strchr(g_pszTable, pszPassword[i]);
		char* pI1 = strchr(g_pszTable, pszPassword[i+1]);

		if(!pI || !pI1)
		{
			bIsValidString = FALSE;
			break;
		}
		
		if(i == 0)
		{
			b = 259 - (((int)(strlen(g_pszTable) - strlen(pI1))/16) | ((strlen(g_pszTable) - strlen(pI))*4));
			z = b - 4;
			b = b + (int)(b/3);
		}

		szDecrypted[nIndex] = (char)(((strlen(g_pszTable) - strlen(pI))*4) | (int)((strlen(g_pszTable) - strlen(pI1))/16));
		nIndex++;
		
		if(i+2 > b)
			break;

		char* pI2 = strchr(g_pszTable, pszPassword[i+2]);
		if(!pI2)
		{
			bIsValidString = FALSE;
			break;
		}

		szDecrypted[nIndex] = (char)( (  (int)((strlen(g_pszTable) - strlen(pI2))/4) & 15) | ( ((strlen(g_pszTable) - strlen(pI1)) & 15)*16) );
		nIndex++;

		if(i+3 > b)
			break;

		char* pI3 = strchr(g_pszTable, pszPassword[i+3]);
		if(!pI3)
		{
			bIsValidString = FALSE;
			break;
		}

		szDecrypted[nIndex] = (char)( ((strlen(g_pszTable) - strlen(pI3)) & 63) | (((strlen(g_pszTable) - strlen(pI2)) & 3)*64));
		nIndex++;
		
		i += 4;
		nLoopCount++;
	}

	if(bIsValidString && z)
	{
		char* pszEncode = new char[z+1];
		memset(pszEncode, 0, z+1);

		int nLen = strlen(szDecrypted);
		for(i=0; i<z; i++)
			pszEncode[i] = (char)(szDecrypted[nLen - i - 1] ^ 90);

		char szMessage[1024];
		memset(szMessage, 0, 1024);
		strcpy(szMessage, "Encrypted string: \t");
		strcat(szMessage, pszPassword);
		strcat(szMessage, "\r\nDecrypted string: \t");
		strcat(szMessage, pszEncode);
		strcat(szMessage, "\r\n---------------------------------------------------");

		PostLastMessage(szMessage);

		delete [] pszEncode;
	}
}